#include "AIMNetThread.h"

//---------------------------------------------------------------------

void AIMNetThread::SendPacket( DataContainer packet, char channel ) {

	DataContainer sendData;
	
	// tack on the FLAP header
	sendData << '\x2a';
	sendData << channel;
	sendData << ToAIMWord( sendSequence );
	sendData << ToAIMWord( packet.length() );
	
	// increment the sequence number
	if( sendSequence == 0xFFFF )
		sendSequence = 0;
	else
		++sendSequence;	

	// Now add the packet itself
	sendData << packet;

	// bon voyage...
	Transmit( sendData );
}

//---------------------------------------------------------------------

void AIMNetThread::SendSNACPacket( SNAC_Object& obj ) {

	// SNAC's are always send on channel 0x02
	SendPacket( obj.Packet(), 0x02 );
}

//---------------------------------------------------------------------

void AIMNetThread::Transmit( DataContainer packet ) {

	// dump the packet
	if( debugDumpOut ) {
		printf( "\n[OUTGOING PACKET (%u bytes)]\n", packet.length() );
		Dump( packet );
		printf( "--------------]\n\n" );
	}

	//socket_send( packet.c_ptr(), packet.length() );
	SendData( packet.c_ptr(), packet.length() );
}


//---------------------------------------------------------------------

void AIMNetThread::Dump( DataContainer& packet ) {

	// dump it...
	for( short i = 0; i < packet.length(); ++i ) {
		printf( "%02X%s", (unsigned char)packet[i], " " );
		if( !((i+1) % 8) && ((i+1) % 16) )
			printf( "  " );
		if( !((i+1) % 16) )
			printf( "\n" );
	}
	printf( "\n" );
}

//---------------------------------------------------------------------
// The Receive() function assumes that in the event we only get a part of
// the 6-byte FLAP header, then we'll get the remainder in the next packet...
// perhaps this is sloppy programming but that seems like a safe assumption.

char AIMNetThread::Receive() {

	int recvCount;
	char channel;
	unsigned short seq, flapLen, wantedSize;
	char* dataTemp = NULL;
	bool newMode = false;
	char flapTemp[6];
	
	// Here, the thinking goes like this... if recvBuf.length() is the same as the correct
	// size of the (full, untruncated) packet, it's OK to dump it and get a new one to
	// process. Otherwise we have to wait until we get the full packet from the last time
	// around. Hope this all works...
	
	// if we got the entire last packet, grab the next FLAP header
	if( recvBuf.length() == correctFLAPSize ) 
	{
		// Try and get the 6-byte flap header (or what's left of it)
		recvCount = ReadData( flapTemp, 6 - flapRead );
		for( unsigned short i = flapRead; i < (flapRead + recvCount); i++ )
			flapHeader[i] = flapTemp[i-flapRead];

		// Did we get the entire header?
		if( recvCount + flapRead == 6 ) {
			flapRead = 0;
		} else {
			printf( "got only part of the flap header: %d bytes\n", recvCount );
			flapRead = recvCount;
			return -1;
		}

		// get 2 byte FLAP_id, presumably 0x2aXX
		recvBuf = "";
		if( flapHeader[0] != 0x2a )
			printf( "BIG WARNING!! FLAP start byte NOT correct! (!= 0x2a)\n" );
		channel = flapHeader[1];	
		printf( "start data: %c %c\n", flapHeader[0], flapHeader[1] );

		// get the server's sequence number
		seq = GetWord( (char*)(flapHeader+2) );
		printf( "sequence number: %u\n", seq );

		// check the sequence numbers
		if( !recvSequence )
			recvSequence = seq;
		else if( seq != ++recvSequence ) {

			// odd... this packet was out of order. Oh well.
			printf( "***WARNING: received packet out of order!\n" );
		}

		// Get the FLAP data length (the last 2 bytes of the header)
		flapLen = GetWord( (char*)(flapHeader+4) );
		correctFLAPSize = flapLen;
		printf( "FLAP data length: %u\n", flapLen );

		// clear out recvBuf, so this next calculation works neatly
		recvBuf = "";
		newMode = true;
	}

	// This is how much we want to be able to read... either the whole packet,
	// or the remainder of the chunk that's missing from the current packet
	wantedSize = correctFLAPSize - recvBuf.length();
	
	// Now, make a buffer and try to grab that much data
	dataTemp = new char[wantedSize];
	recvCount = ReadData( dataTemp, wantedSize );
	
	// if this is a new packet, *replace* recvBuf with our new data
	if( newMode )
		recvBuf = DataContainer( dataTemp, recvCount );
		
	// otherwise, *append* recvBuf with our new data
	else recvBuf << DataContainer( dataTemp, recvCount );
	
	// clear out our temp buffer	
	delete[] dataTemp;
	
	// now... do we have a complete packet for processing?
	// if not, return -1 (which means no processing will occur)
	if( recvBuf.length() != correctFLAPSize )
		return -1;

	// dump the packet
	if( debugDumpIn ) {
		printf( "\n[INCOMING PACKET (%u bytes, no FLAP header)]\n", recvBuf.length() );
		Dump( recvBuf );
		printf( "--------------]\n\n" );
	}

	// since all is apparently well, return the channel...
	channel = flapHeader[1];
	return channel;
}

//---------------------------------------------------------------------

void AIMNetThread::SendAppMessage( BMessage* msg ) {
	BMessenger sender( APP_SIGNATURE );
	sender.SendMessage( msg );
	delete msg;
}

//---------------------------------------------------------------------