#include <ctype.h>
#include "AIMNetThread.h"
#include "Globals.h"
#include "constants.h"
#include "Say.h"

bool debugDumpIn = true;
bool debugDumpOut = true;

//---------------------------------------------------------------------

// Constructor
AIMNetThread::AIMNetThread()
			 :TCPThread( "aim_net_thread" )
{
	preLoginPhase = 0;			// not doing the prelogin yet
	connected = false;
	
	requestID = 0x2720;
	flapRead = 0;
	correctFLAPSize = 0;
}

//---------------------------------------------------------------------

// Close
//void AIMNetThread::Close()
//{
//	Logout();
//	TCPThread::Close();
//}

//---------------------------------------------------------------------

// Logout function
void AIMNetThread::Logout() {

	printf( "logging out\n" );
	user->SetLoggedIn( false );
	
	DataContainer blank;

	// sending a blank packet on channel 0x04 is apparently the logout command
	SendPacket( blank, 0x04 );
	Close();
}

//---------------------------------------------------------------------

// Data waiting function
void AIMNetThread::DataWaiting() {

	// if we are still logging in, redirect to the login procedure
	if( preLoginPhase ) {
		DoLogin();
		return;
	}

	// otherwise, it's just a standard AIM command/message
	ProcessCommands();
}

//---------------------------------------------------------------------

// Function to update the status of the logon
void AIMNetThread::LoginStep( const char* message, bool updateText ) {

	BMessenger sender( APP_SIGNATURE );
	BMessage* sendMessage;

	// Stick the message into a BMessage and send it
	sendMessage = new BMessage( BEAIM_LOGIN_STEP );
	sendMessage->AddString( "status", message );
	sendMessage->AddBool( "update_text", updateText );
	sender.SendMessage( sendMessage );
}

//---------------------------------------------------------------------

void AIMNetThread::CancelSignOn() {
	
	LoginFailure( "Login was canceled." );
}

//---------------------------------------------------------------------

void AIMNetThread::MessageReceived( BMessage* msg ) {

	switch( msg->what ) {
		
		case BEAIM_SIGN_ON:
			Login( msg );
			break;
			
		case BEAIM_LOGOUT:
			Logout();
			break;
			
		case BEAIM_SEND_MESSAGE:
			SendMessage( msg );
			break;
			
		case BEAIM_GET_PERSON_INFO:
			RequestPersonInfo( msg );
			break;
			
		case BEAIM_GET_AWAY_INFO:
			RequestAwayInfo( msg );
			break;
			
		case BEAIM_CANCEL_SIGN_ON:
			CancelSignOn();
			break;
			
		case BEAIM_SIGN_OFF:
			Logout();
			break;
			
		case BEAIM_SEND_ARBITRARY_DATA:
			SendArbitraryData( msg );
			break;
			
		case BEAIM_NOW_IDLE:
			SendIdleMessage( true );
			break;
			
		case BEAIM_NOW_ACTIVE:
			SendIdleMessage( false );
			break;

		case BEAIM_SEND_IDLE_PULSE:
			SendIdlePulse();
			break;

		case BEAIM_PEOPLELIST_COMMIT:
			UpdateServerBuddyList( msg );
			break;

		case BEAIM_SEARCH_BY_EMAIL:
			SearchByEmail( msg );
			break;
			
		case BEAIM_GOING_AWAY:
			SetAwayStatus( msg );
			break;
			
		case BEAIM_WARN_SOMEONE:
			SendWarning( msg );
			break;
			
		case BEAIM_SET_USER_BLOCKINESS:
			SetUserBlockiness( msg );
			
		default:
			TCPThread::MessageReceived(msg);
			break;
	}
}

//---------------------------------------------------------------------

void AIMNetThread::LoadBuddyList( GenList<DataContainer>& buddies ) {
	
	char* blist = NULL;
	BString blist2;
	list->GetUserList( blist2 );
	blist = const_cast<char*>(blist2.String());
	DataContainer adder;
	char *i = blist, *q = blist;
	const int maxList = 50;
	char name[50];
	int l = 1, count = 0;
	
	if( blist ) {
		while( *i ) {
			if( *i == 9 ) {
				strncpy( name, q, l );
				name[l-1] = '\0';
				//buddies << char(l-1) << name;
				adder << char(l-1) << name;
				if( !(++count % maxList) ) {
					buddies.Add( adder );
					adder = "";
				}
									
				l = 0;
				q = i+1;
			}
			++i; ++l;
		}
		if( i != q ) {
			//buddies << char(l-1) << q;
			adder << char(l-1) << q;
			buddies.Add( adder );
		}
	}	
}

//---------------------------------------------------------------------

void AIMNetThread::Disconnected() {

	// if connected is set to false BEFORE this function is called, it was
	// most likely expected. In that case, we don't need to notify the user.
	if( connected ) {
		printf( "disconnected. Darn.\n" );
		
		// were we logging in when this happened?
		if( preLoginPhase )
			LoginFailure( "Login failed... you were disconnected from the server." );
	}

	SendAppMessage( new BMessage(BEAIM_DISCONNECTED) );
	connected = false;
	Close();
}

//---------------------------------------------------------------------

void AIMNetThread::Connected() {

	printf( "piffleconnected\n" );

	connected = true;
	DoLogin();
}

//---------------------------------------------------------------------

/*
void AIMNetThread::HandleError( int error, const char* message ) {

	Say( "Error() called" );
	
	switch( error ) {

		case TCP_INTERNALERROR:
			Say( "TCP_INTERNALERROR" );
			break;

		case TCP_COULDNOTCONNECT:
			LoginFailure( "Could not connect to the server." );
			break;

		case TCP_COULDNOTRESOLVEHOSTNAME:
			Say( "TCP_COULDNOTRESOLVEHOSTNAME" );
			break;

		case TCP_UNKNOWNERROR:
			Say( "TCP_UNKNOWNERROR" );
			break;	
	}
}
*/

//---------------------------------------------------------------------
