#include "AIMNetThread.h"
#include "AIMConstants.h"

//---------------------------------------------------------------------

bool AIMNetThread::ProcessCommands() {

	// Get the actual data
	char channel = Receive();
	SNAC_Object snac;

	// Make a SNAC object out of the received data, and decode it
	if( channel != -1 ) {
		snac = SNAC_Object( recvBuf );
		DecodeSNACPacket( snac );
	}
		
	return false;
}

//---------------------------------------------------------------------

DataContainer AIMNetThread::ToAIMWord( unsigned short word ) {

	DataContainer ret = "";
	ret << char( word >> 8 );
	ret << char( word & 0xFF );
	
	return ret;
}

//---------------------------------------------------------------------

DataContainer AIMNetThread::ToTLV( unsigned short id, unsigned short len, DataContainer data )
{
	// Make a DataContainer containing a TLV
	DataContainer ret = "";
	ret << ToAIMWord( id );
	ret << ToAIMWord( len );
	ret << data;

	return ret;
}

//---------------------------------------------------------------------

// assumes a 2-byte input
unsigned short AIMNetThread::GetWord( char* input ) {
	
	unsigned short ret = 0;

	// Get the correct byte order
	ret =  ( (( (unsigned short) input[0]) & 0xFF) << 8);
	ret +=    ( (unsigned short) input[1]) & 0xFF;
	
	return ret;
}

//---------------------------------------------------------------------

// i is the value at which the word starts in input
unsigned short AIMNetThread::GetWord( DataContainer input, short i ) {
	
	unsigned short ret = 0;	
	
	// Get the correct byte order
	ret =  ( (( (unsigned short) input[i+0]) & 0xFF) << 8);
	ret +=    ( (unsigned short) input[i+1]) & 0xFF;
	
	return ret;
}

//---------------------------------------------------------------------

void AIMNetThread::DecodeSNACPacket( SNAC_Object& snac ) {

	printf( "SNAC packet received...  genus: %X   specie: %X\n", snac.genus, snac.specie );

	// send each genus to the appropriate sub-handlers
	switch( snac.genus ) {
		
		case GENERIC_SERVICE_GENUS:
			DecodeGenericServiceSNAC( snac );
			break;
			
		case LOCATION_SERVICE_GENUS:
		case USER_LOOKUP_GENUS:
			DecodeUserSNAC( snac );
			break;
		
		case BUDDY_LIST_GENUS:
			DecodeBuddyListSNAC( snac );
			break;
			
		case BOS_SPECIFIC_GENUS:
			DecodeBOSSNAC( snac );
			break;
			
		case ADMINISTRATIVE_GENUS:
			DecodeAdministrativeSNAC( snac );
			break;
			
		case MESSAGING_GENUS:
			DecodeMessagingSNAC( snac );
			break;
			
		case INVITATION_GENUS:
		case STATS_GENUS:
		case POPUP_NOTICE_GENUS:
			DecodeOtherSNAC( snac );
			break;
			
		default:
			printf( "**********************\nWhoa... Unknown GENUS!  Gen: %X	Spc: %X\n**********************", snac.genus, snac.specie );
	}
}

//---------------------------------------------------------------------

void AIMNetThread::DecodeUserSNAC( SNAC_Object& snac ) {

	// snac from the location service
	if( snac.genus == LOCATION_SERVICE_GENUS ) {
	
		switch( snac.specie ) {
	
			// error... user not online
			case 0x01:
				printf( "LOCATION_SERVICE_GENUS error! (uh-oh)\n" );
				FailedPersonInfo( snac );
				break;
				
			// rights response
			case 0x03:
				printf( "location rights response received\n" );
				break;
				
			// user information
			case 0x06:
				printf( "user information received\n" );
				ReceivePersonInfo( snac );
				break;
				
			// watcher notification (?)
			case 0x08:
				printf( "watcher notification (?) received\n" );
				break;
				
			default:
				printf( "**********************\nUnknown Packet!  Gen: %X	Spc: %X\n**********************", snac.genus, snac.specie );
				
		}
	}
	
	// snac from the user lookup service
	if( snac.genus == USER_LOOKUP_GENUS ) {
	
		switch( snac.specie ) {
	
			// error
			case 0x01:
				printf( "USER_LOOKUP_GENUS error! (usually: search failed)\n" );
				ReceiveFailedSearch( snac );
				break;
				
			// response to search by email address
			case 0x03:
				printf( "response to search by email address\n" );
				ReceiveSearchResults( snac );
				break;
				
			default:
				printf( "**********************\nUnknown Packet!  Gen: %X	Spc: %X\n**********************", snac.genus, snac.specie );
				
		}
	}
	
}
		
//---------------------------------------------------------------------	
		
void AIMNetThread::DecodeBuddyListSNAC( SNAC_Object& snac ) {

	switch( snac.specie ) {

		// error
		case 0x01:
			printf( "BUDDY_LIST_GENUS error! (uh-oh)\n" );
			break;

		// buddy list rights response
		case 0x03:
			printf( "buddy list rights response\n" );
			break;
			
		// watcher list (?) response
		case 0x07:
			printf( "Watcher list response\n" );
			break;
			
		// watcher (?) notification
		case 0x09:
			printf( "Watcher (?) notification\n" );
			break;
			
		// reject (?) notification
		case 0x0A:
			printf( "reject (?) notification\n" );
			break;
			
		// oncoming buddy/buddy refresh
		case 0x0B:
			printf( "oncoming buddy\n" );
			BuddyOnline(snac);
			break;
			
		// offgoing buddy
		case 0x0C:
			printf( "offgoing buddy\n" );
			BuddyOffline(snac);
			break;
			
		default:
			printf( "**********************\nUnknown Packet!  Gen: %X	Spc: %X\n**********************", snac.genus, snac.specie );
	}
}

//---------------------------------------------------------------------

void AIMNetThread::DecodeMessagingSNAC( SNAC_Object& snac ) {

	switch( snac.specie ) {

		// error
		case 0x01:
			printf( "MESSAGING_GENUS error! (uh-oh)\n" );
			FailedMessage(snac);
			break;
			
		// ICBM parameter info response
		case 0x05:
			printf( "ICBM parameter info response\n" );
			break;
			
		// incoming IM
		case 0x07:
			printf( "incoming IM!\n" );
			ReceiveMessage(snac);
			break;
			
		// response to a warning-request
		case 0x09:
			printf( "response to a warning-request\n" );
			break;

		// missed messages
		case 0x0A:
			printf( "some messages were missed... the other person sent them too fast.\n" );
			break;			

		// client error (?)
		case 0x0B:
			printf( "client error... huh?\n" );
			break;
			
		// host acknowledgement
		case 0x0C:
			printf( "Host ack...\n" );
			break;
			
		default:
			printf( "**********************\nUnknown Packet!  Gen: %X	Spc: %X\n**********************", snac.genus, snac.specie );			
	}
}
		
//---------------------------------------------------------------------
		
void AIMNetThread::DecodeAdministrativeSNAC( SNAC_Object& snac ) {

	switch( snac.specie ) {

		// error
		case 0x01:
			printf( "ADMINISTRATIVE_GENUS error! (uh-oh)\n" );
			break;
			
		// information reply
		case 0x03:
			printf( "admin information reply\n" );
			break;
		
		// change information reply
		case 0x05:
			printf( "admin change information reply\n" );
			break;
			
		// account confirm reply
		case 0x07:
			printf( "admin account confirm reply\n" );
			break;
			
		// account delete reply
		case 0x09:
			printf( "admin account delete reply\n" );
			break;			
			
		default:
			printf( "**********************\nUnknown Packet!  Gen: %X	Spc: %X\n**********************", snac.genus, snac.specie );			
	}
}
		
//---------------------------------------------------------------------

void AIMNetThread::DecodeBOSSNAC( SNAC_Object& snac ) {

	switch( snac.specie ) {

		// error
		case 0x01:
			printf( "BOS_SPECIFIC_GENUS error! (uh-oh)\n" );
			break;
			
		// BOS rights response
		case 0x03:
			printf( "BOS rights response\n" );
			break;
		
		// another error
		case 0x09:
			printf( "server BOS error... odd...\n" );
			break;
			
		default:
			printf( "**********************\nUnknown Packet!  Gen: %X	Spc: %X\n**********************", snac.genus, snac.specie );
			
	}
}
		
//---------------------------------------------------------------------

void AIMNetThread::DecodeOtherSNAC( SNAC_Object& snac ) {

	// response to the "invite a friend" command
	if( snac.genus == INVITATION_GENUS && snac.specie == 0x03 ) {
		printf( "Invite a friend to join AIM acknowledged\n" );
		return;
	}
	
	// Popup notices
	else if( snac.genus == POPUP_NOTICE_GENUS ) {
		
		// error
		if( snac.specie == 0x01 ) {
			printf( "POPUP_NOTICE_GENUS error! (uh-oh)\n" );
			return;
		}
		
		// display popup
		else if( snac.specie == 0x02 ) {
			printf( "server wants to display a popup\n" );
			return;
		}
		
		else {
			printf( "**********************\nUnknown Packet!  Gen: %X	Spc: %X\n**********************", snac.genus, snac.specie );
			return;
		}
	}
	
	// Stats
	else if( snac.genus == STATS_GENUS ) {
		
		// error
		if( snac.specie == 0x01 ) {
			printf( "STATS_GENUS error! (uh-oh)\n" );
			return;
		}
		
		// set minimum report interval
		else if( snac.specie == 0x02 ) {
			printf( "set minimum report interval\n" );
			return;
		}
		
		// stats report ack
		else if( snac.specie == 0x04 ) {
			printf( "stats report ack\n" );
			return;
		}
		
		else {
			printf( "**********************\nUnknown Packet!  Gen: %X	Spc: %X\n**********************", snac.genus, snac.specie );
			return;
		}
	}
}

//---------------------------------------------------------------------
		
void AIMNetThread::DecodeGenericServiceSNAC( SNAC_Object& snac ) {

	// in case it's needed...
	DataContainer temp;

	switch( snac.specie ) {

		// error
		case 0x01:
			printf( "GENERIC_SERVICE_GENUS error! (uh-oh)\n" );
			break;

		// server ready
		case 0x03:
			temp << ToAIMWord(0x0001) << ToAIMWord(0x0002)
				 << ToAIMWord(0x0003) << ToAIMWord(0x0004)
				 << ToAIMWord(0x0006) << ToAIMWord(0x0008)
				 << ToAIMWord(0x0009) << ToAIMWord(0x000a)
				 << ToAIMWord(0x000b) << ToAIMWord(0x000c);
			if( snac.data == temp )
				printf( "Server ready response verified OK.\n" );
			else
				printf( "Whoops...Server ready response isn't right!\n" );
			break;
					
		// rate information received
		case 0x07:
			printf( "Rate information received\n" );
			break;
					
		// rate violation (bad)
		case 0x0a:
			printf( "Rate violation!\n" );
			break;
					
		// info on our screen name
		case 0x0f:
			printf( "screen name information received.\n" );
			ParseMyUserInfoPacket( snac.data );
			break;
					
		// warning
		case 0x10:
			printf( "Someone just warned you!\n" );
			ReceiveWarning( snac );
			break;

		// migration notice
		case 0x12:
			printf( "Migration notice... oscar wants us to move!\n" );
			break;
				
		// Message of the day (?) server sends this after BOS login...
		case 0x13:
			printf( "...\n" );
			break;
			
		default:
			printf( "**********************\nUnknown Packet!  Gen: %X	Spc: %X\n**********************", snac.genus, snac.specie );			
	}
}

//---------------------------------------------------------------------

void AIMNetThread::LogMessageReqID( BString userID, int type ) {

	userRequestID rid;

	printf( "Logging Message RID: %d\n", requestID );

	// if the list has 20+ items in it, shuffle one of 'em out
	if( pendingNRRequests.Count() >= 20 )
		pendingNRRequests.Dequeue( rid );

	// add an object to the pending list
	strcpy( rid.userid, userID.String() );
	rid.reqID = requestID++;
	rid.type = type;
	pendingNRRequests.Enqueue( rid );
}

//---------------------------------------------------------------------
