#ifndef _NET_THREAD_BASE_
#define _NET_THREAD_BASE_

#include <Looper.h>

// This is the abstract base class for the net thread's for all protocols... regardless
// of whether they use TCP or UDP. It just defines the basic interface for the net thread
// stuff... most of the actual protocol stuff will be via BMessage's, so all this class
// needs to deal with is the actual communication interface.

class NetThreadBase : public BLooper {

	public:
		NetThreadBase( char* name ) : BLooper( name, B_LOW_PRIORITY ) {};

		// Generalized virtual functions
		virtual void Close() = 0;
		virtual void Login( BMessage* ) = 0;
		virtual void Logout( BMessage* ) = 0;
		
	protected:

		// Generalized pure virtual functions
		virtual bool IsConnected() = 0;
		virtual bool Connect( char*, int32, bool=true ) = 0;
		virtual void Disconnect() = 0;
		virtual void MessageReceived( BMessage* ) = 0;

		// Handler functions (don't have to be overridden)
		virtual void Connected() {};
		virtual void Disconnected() {};
		virtual void DataWaiting() {};
		virtual void HandleError( int error, const char* message = 0 ) {};
};

#endif