#ifndef _PEOPLE_DATA_H_
#define _PEOPLE_DATA_H_

#include <stdio.h>
#include <OutlineListView.h>
#include <ListItem.h>
#include "GenList.h"
#include "constants.h"
#include "MiscStuff.h"

struct GroupInfo;
struct PersonInfo;

//=========================================================================

class PeopleSetupItem : public BListItem
{ 
	public: 
		PeopleSetupItem( BString user, bool group );
		~PeopleSetupItem(); 
		virtual void DrawItem( BView *owner, BRect frame, bool complete = false );
		BString UserName();
		void SetUserName( BString un );
		bool isGroup() { return isgroup; };

	private: 
		BString userName;
		bool isgroup;
};

//=========================================================================

class GroupItem : public BListItem
{ 
	public: 
		GroupItem( BString s, unsigned int on = 0, unsigned int tot = 0 );
		~GroupItem(); 
		virtual void DrawItem( BView *owner, BRect frame, bool complete = false ); 
		BString GroupName();
		void  SetGroupName( BString );		
		void SetNumbers( unsigned int on, unsigned int tot );

	private: 
		BString name; 
		unsigned int online, total;
};

//=========================================================================

class PeopleItem : public BListItem
{ 
	public: 
		PeopleItem( BString user, int32 stat = BS_ACTIVE );
		~PeopleItem(); 
		virtual void DrawItem( BView *owner, BRect frame, bool complete = false ); 
		BString UserName();
		void SetUserName( BString );
		void SetStatus( unsigned int );
		void SetWarningLevel( unsigned int );

	private: 
	
		void MakeDisplayName();
	
		BString user;
		BString displayname;
		unsigned int status;
		unsigned int warningLevel;
		static BBitmap* awayIcon;
};

//=========================================================================

struct PersonCapsule {
	BString name;
	bool add;
};

//=========================================================================

struct PersonInfo {

	// trash the display items
	~PersonInfo() {
		if( item )
			delete item;
		if( setupitem )
			delete setupitem;
	}

	PeopleItem* item;
	PeopleSetupItem* setupitem;
	GroupInfo* parent;
	BString name;
	unsigned int status;
	unsigned int warningLevel;
	
	bool operator==( PersonInfo& p ) {
		//if( strcasecmp(p.name,name) == 0 )
		//	return true;
		if( CompareScreenNames( const_cast<char*>(p.name.String()), const_cast<char*>(name.String()) ) )
			return true;
		return false;
	}
};

//=========================================================================


struct GroupInfo {
	
	// trash everything in the list
	~GroupInfo() {
		PersonInfo* del;
		while( names.Pop(del) )
			delete del;
	}

	GroupItem* item;
	PeopleSetupItem* setupitem;
	unsigned int on;
	unsigned int total;
	BString group;
	GenList<PersonInfo*> names;	
};

//=========================================================================

// structure to handle timed list updates
struct PersonChange {
	PersonInfo* person;
	unsigned int ChangeTo;
	unsigned int Counter;
};

//=========================================================================

class PeopleList {

	public:
		PeopleList();
		~PeopleList();
		void SetDisplayList( BOutlineListView* list );
		void SetSetupList( BOutlineListView* list );
		//void LoadUserSettings( char* user );
		
		void Clear();
		
		PersonInfo* AddPerson( BString user, BString group, bool commitnow=false, bool check=true );
		void SetOnline( BString user, unsigned int status );
		void SetOnline( PersonInfo*, unsigned int status );
		void SetOffline( BString user );
		void SetOffline( PersonInfo* );
		void RemovePerson( BString user, bool commitnow=false );
		void RemovePerson( PersonInfo*, unsigned int groupIndex, bool commitnow=false );
		PersonInfo* FindPerson( BString userName, unsigned int* gi = NULL );	
		void ChangePerson( BString oldUser, BString newUser, BString group );
		void ChangePerson( PersonInfo*, unsigned int groupIndex, BString newUser, BString group );	
		void SetPersonStatus( BString user, unsigned int, int32 = -1, unsigned int = 0 );
		void SetPersonStatus( PersonInfo*, unsigned int,  int32 = -1, unsigned int = 0 );

		void UpdatePersonNameFormat( BString user );

		GroupInfo* FindGroup( BString group, bool make = false, unsigned int* gi = NULL );
		GroupInfo* AddGroup( BString group, bool check = true );
		void RemoveGroup( BString group );
		void ChangeGroup( BString, BString );
		unsigned int CountGroups();
		void GetGroup( unsigned int, BString& );
		
		void AddCommitListItem( BString user, bool add );
		void DispatchCommitList( GenList<PersonCapsule>&, bool add );
		void ClearCommitList();
		void Commit();
		void UpdateChangeList();
		
		void Attach( PersonInfo*, BString, bool make = false );
		void Attach( PersonInfo*, GroupInfo* );
		void Detach( PersonInfo*, unsigned groupIndex );
		
		void GetGroupInfo( BString, unsigned int&, unsigned int& );
		void GetUserList( BString& list );
		
		unsigned int CountPeople();
		void GetPersonInfo( BString user, PersonInfo*& );
		
		// iterators
		bool GetFirstGroup( BString& );
		bool GetNextGroup( BString& );
		bool GetFirstBuddyForCurrentGroup( BString& );
		bool GetNextBuddyForCurrentGroup( BString& );
		
		void Debug();
		
		//void SetUserName( char* username );
		
		//void Load();
		//void Save();
	
	private:
		
		void SetupChange( BString user, unsigned int changeTo );
		void SetupChange( PersonInfo* person, unsigned int changeTo );
		void CorrectGroupNumbers( GroupInfo*, unsigned int status, bool add );
		bool isOnline( const uint32 );
		
		bool FixChangesForStatus( PersonInfo* person, unsigned int status );
	
		//char UserName[DISPLAY_NAME_MAX];
	
		BOutlineListView* display;
		BOutlineListView* setup;
		GenList<PersonCapsule> commitList;
		GenList<PersonChange> changeList;
		GenList<GroupInfo*> main;		
				
		//FILE *ptr;
		BLocker* lock;
		unsigned int commitAdds;
};


#endif
