#ifndef _TCP_THREAD_H_
#define _TCP_THREAD_H_

#include <Looper.h>
#include <Invoker.h>
#include <kernel/OS.h>
#include <net/socket.h>
#include <netinet/in.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>
#include <errno.h>
#include <String.h>

// ERROR VALUES
const int32 TCP_ERROR_DNS = 1;  // Error when DNS lookup fails
const int32 TCP_ERROR_NO_CONNECT = 2;  // Error when connection fails


// BMESSAGES

// Incoming (from user)
const uint32 TCP_CONNECT = 'TCPc';
/* Define TCP_CONNECT:
	String: HOSTNAME - hostname to connect to
	Int32: PORT - port to connect on
*/

const uint32 TCP_DISCONNECT = 'TCpD';
/* Disconnects the thread. No other data needed. */


const uint32 TCP_DATA_SEND = 'TCDs';
/* Define TCP_DATA_SEND:
	String: DATA - data to send
	Int32: LENGTH - length of data
*/

// Outgoing (from TCPThread)
const uint32 TCP_ERROR = 'TErr';
/* Define TCP_ERROR:
	Int32: TYPE - the error type (TCP_ERROR_DNS, TCP_ERROR_NO_CONNECT, etc)
	String: HOSTNAME - hostname that had the problem
	Int32: PORT - port that had the problem
*/

const uint32 TCP_CONNECTED = 'TCon';
/* Define TCP_CONNECTED:
	String: HOSTNAME - hostname that's connected
	Int32: PORT - port that's connected
*/

const uint32 TCP_DISCONNECTED = 'TDCn';
/* Define TCP_DISCONNECTED:
	String: HOSTNAME - hostname that disconnected
	Int32: PORT - port that disconnected
*/

const uint32 TCP_DATA_RECV = 'TDRc';
/* Define TCP_DATA_RECV:
	String: DATA - actual received data
	Int32: LENGTH - length of data
	String: HOSTNAME - hostname data is from
	Int32: PORT - port data is from
*/

//INTERNAL
const uint32 TCP_RECV = 'TRcv';

class TCPThread : public BLooper, public BInvoker
{
	public:
		TCPThread( char* = "TCPThread" );
		
		// Hooks
		virtual void Logout();

	protected:
		virtual void MessageReceived( BMessage* );
		virtual void Quit( );
		void Connect( char *, int );
		static int32 ConnectThreadEntry( void * );
		int32 ConnectThread( );
		bool StartConnect ( );
		void Close( bool = false );
		void SendStatus( int32, int32 = 0 );
		bool StartRead( );
		static int32 ReadThreadEntry( void * );
		int32 ReadThread( );
		void WakeReadThread( );
		void WaitReadThread( );
		void WaitConnectThread( );
		ssize_t ReadData( void *, size_t );
		ssize_t SendData( const void *, size_t );
		
		// Hooks
		virtual void DataWaiting();
		virtual void Connected();
		virtual void Disconnected();
		virtual void Error( int32 );
		
	private:
		int socketID;
		bool socketConnected;
		BString Hostname;
		int Port;
		thread_id readID;
		thread_id connectID;
		bool waitingToDie;
};

#endif