#ifndef _USER_MANAGER_H_
#define _USER_MANAGER_H_

#include <String.h>
#include <File.h>
#include <Directory.h>
#include <Locker.h>
#include <time.h>
#include <stdio.h>
#include "GenList.h"
#include "WindowManager.h"

//-----------------------------------------------------

struct preLogStruct {
	BString name;
	BString password;
	bool savePassword;
	bool autoLogin;
};
		
//-----------------------------------------------------

struct awayStruct {
	BString name;
	BString message;
};

//-----------------------------------------------------

struct prefInt32Struct {
	BString name;
	int32 val;
};
struct prefBoolStruct {
	BString name;
	bool val;
};
struct prefStringStruct {
	BString name;
	BString val;
};

//-----------------------------------------------------

class UserManager {

	public:
	
		UserManager();
		~UserManager();

		void SetUser( BString user );
		BString User();
		void Clear();

		// profile functions		
		BString Profile();
		void SetProfile( BString );
		
		// warning level stuff
		int32 WarningLevel();
		void SetWarningLevel( int32 );

		// away message functions
		void SetAwayMessage( BString name, BString oldname, BString message );
		bool GetAwayMessage( BString name, BString& message );
		int  FindAwayMessage( BString name );
		void RemoveAwayMessage( BString name );
		bool GetFirstAwayMessage( awayStruct& );
		bool GetNextAwayMessage( awayStruct& );
		void SetAwayMode( int awayMode, BString messageInfo = "" );
		BString CustomAwayMessage();
		BString CurrentAwayMessage();
		BString CurrentAwayMessageName();
		int  AwayMode();
		void ReplaceTagVars( BString& );

		// file functions
		void WriteUserFile();
		void ReadUserFile();
		void WriteGlobalFile();
		void ReadGlobalFile();
		
		// preference functions
		void WriteBool( BString name, bool value, bool=false );
		bool ReadBool( BString name, bool def, bool=false );
		void WriteString( BString name, BString val, bool=false );
		void ReadString( BString name, BString& buf, BString def, bool=false, bool=true );
		void WriteInt32( BString name, int32 value, bool=false );
		int32 ReadInt32( BString name, int32 def, bool=false );
		
		// blocklist
		bool GetNextBlockedUser( BString& user, bool first );
		void BlockUser( BString, bool report = true );
		void UnblockUser( BString );
		bool IsUserBlocked( BString );
		
		// prelogin stuff
		bool GetNextUser( BString&, bool first = false );
		void GetUserLoginParameters( BString user, BString& pw, bool& autoLogin, bool& savePassword );
		void SaveLoginParameters( BString user, BString pw, bool autoLogin, bool savePassword, bool ain=false );
		void SetLastLogin( BString );
		BString LastLogin();
		
		// other stuff
		void SetLoggedIn( bool li );
		bool LoggedIn() { return loggedIn; };
		void SetServerTimeOffset( time_t offset );
		time_t ServerTimeOffset();
		void UnEntitizeNormalString( BString& );

	private:
	
		BString GetUserFileName();
		void WriteAwayMessages( FILE* ptr );
		void WriteBuddyList( FILE* ptr );
		void WritePrefs( FILE* ptr );
		void WritePreLoginData( FILE* ptr );
		void WriteWindowPositions( FILE* ptr );
		void WriteBlockedUsers( FILE* ptr );
		void EntitizeNormalString( BString& );
		void EntitizeCDataString( BString& );
		
		// expat XML handler functions
		static void startElement( void *userData, const char *name, const char **atts );
		static void endElement( void *userData, const char *name );
		static void characterData( void *userData, const char *data, int len );
		void ReadOption( const char *name, const char **atts );
		
		unsigned readMode;
		bool readGlobal;
		BString rGroup, rBuddy, rAwayName, rAwayMessage;
		winPosRect wpRect;
		
		// away status and stuff
		GenList<awayStruct> awayMessages;
		BString awayMessageName;
		BString currentAwayMessage;
		BString customAwayMessage;
		int awayMode;
		
		// block list
		GenList<BString> blockList;

		// vars
		BDirectory *prefsDirectory;
		BString currentUser;
		BString profile;
		
		// prelogin vars
		GenList<preLogStruct> preLogins;
		preLogStruct plTemp;
		BString lastLogin;

		// preferences
		GenList<prefInt32Struct> prefInt;
		GenList<prefBoolStruct> prefBool;
		GenList<prefStringStruct> prefString;
		GenList<prefInt32Struct> gPrefInt;
		GenList<prefBoolStruct> gPrefBool;
		GenList<prefStringStruct> gPrefString;
		
		// other vars
		bool loggedIn;
		int32 warningLevel;
		time_t serverTimeOffset;
		BLocker lock;
};

//-----------------------------------------------------

#endif
