#ifndef _WINDOW_MANAGER_H_
#define _WINDOW_MANAGER_H_

#include <Window.h>
#include <Message.h>
#include <Locker.h>
#include "GenList.h"
#include "constants.h"
#include "LoginBox.h"
#include "DataSender.h"

// holds window positions
struct winPosRect {
	BString user;
	BRect frame;
	float divider;
};


// Used to define the list of open IM windows
struct IMWindowItem {
	IMWindowItem() { window = NULL; };
	IMWindowItem( char* sn, int tp, BWindow* w ) {
		strcpy( name, sn );
		window = w;
		type = tp;
	};
	char name[INTERNAL_NAME_MAX];
	int type;
	BWindow* window;
};


// WindowManager - the class that handles the opening and closing of various
//   BeAIM windows, like the IM Windows.
class WindowManager {

	public:
		WindowManager();
		
		void OpenBuddyList( BPoint* point = NULL );
		void OpenSignOnWindow( BPoint* point = NULL );
		void CloseSignOnWindow( BPoint* point = NULL );
		void DisplayMessage( BMessage* );
		void DisplayError( BMessage* );
		void MakeDataSenderWindow();
		
		BWindow* GetBuddyList();
		BWindow* OpenUserWindow( BMessage* message );
		void UserWindowOpened( BMessage* message );
		void UserWindowClosed( BMessage* message );
		void SwitchUserWindow( BMessage* message );
		void ForwardIncomingMessage( BMessage* message );
		void BroadcastMessage( BMessage* message );
		
		void ForwardEmailSearchResults( BMessage* message );
		void OpenSingleWindow( BMessage* msg, int32 whichWindow = 0 );
		void CloseSingleWindow( BMessage* msg );
		void SingleWindowClosed( BMessage* msg );
		void SendSingleWindowMessage( BMessage* msg, int );

		void SendBuddyListMessage( BMessage* message );
		void SendSignOnMessage( BMessage* message );
		
		void MakeDialogFrame( BRect&, BPoint );
		void MakeDialogFrame( BRect&, BWindow* );
		void CenterFrame( BRect& );
		
		void ToggleHidden( bool forceShow );
		
		void SetWindowPos( winPosRect& );
		bool GetNextWindowPos( winPosRect&, bool );
		void ClearWindowPositions();
		void CloseDaWindows();
		
		void OpenInputWindow( char* title, char* label, BMessage* = NULL, BWindow* window = NULL,
							  char* initial = NULL, int maxLen = 0, bool toApp = false, char* fname = "value" );

	private:
	
		void CorrectFrame( BRect& );
	
		void CalcNewIMRect();
		void SaveWindowPos( BString user, BRect frame, float divider );
		BWindow* MakeMessageWindow( BString userName );
		BWindow* MakeInfoWindow( BString userName, BWindow* posWindow = NULL );
	
		GenList<winPosRect> winPositions;
	
		BRect IMWindowRect;
		GenList<IMWindowItem> IMWindows;
		BWindow* bList;
		BWindow* signOn;
		BWindow* resWin;
		BWindow* awayEd;
		BWindow* profEd;
		BWindow* custEd;
		BWindow* prefsWin;
		BWindow* impWin;
		BWindow* aboutBox;
		BWindow* blockEd;
		BLocker* lock;
};

#endif
